#include <sys/time.h>

#include <Xm/ScrolledW.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#include <libdlg.h>
#include <libprg.h>
#include <Array.h>

#include "libanalyze.h"
#include "signal.h"
#include "card.h"

#include <icons/save.xpm>
#include <icons/load.xpm>
#include <icons/bomb.xpm>
#include <icons/plot.xpm>
#include <icons/stop.xpm>
#include <icons/card.xpm>
#include <icons/tv.xpm>
#include <icons/help2.xpm>

#define SAMPLES 1000


struct PARAMETER
{
  int          LPTBase;
  ANALYZER    *Module;
  SIGNAL      *Channel[24];
  int          SizeX, SizeY;
  int          Size;
  int         *In, *Out;
  int          Pos;
  int          Delay;
  bool         Buffered;
  Widget       ScrollWin;
  Widget       Form;
  Widget       widget;
  Window       Win;
  XtIntervalId Timer;
};


static int BaseAddr[] = 
{
  0x278,
  0x2F8,
  0x378,
  0x3F8
};


static char Bit4Code[] = 
{
  '0', '1', '2', '3', '4', '5', '6', '7', 
  '8', '9', ' ', ';', ' ', '=', ' ', '?'
};


static PARAMETER Data =
{
  2,
  NULL, 
  {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  600, 400,
  0, 
  NULL, NULL,
  0,
  50,
  FALSE,
  NULL, NULL,
  None, None,
  0
};


static void Stop ();


void Step (int Index)
{
  int count, Value;

  if (Index < 0 || Index >= SAMPLES)
    return;

  Data.Module->Write (Data.Out[Index]);
  Data.In[Index] = Data.Module->Read ();

  Value = Data.In[Index];
  for (count = 0; count < 12; count++)
  {
    Data.Channel[count]->AddValue (Value & 1);
    Value >>= 1;
    if (! Data.Buffered) 
      Data.Channel[count]->Refresh ();
  }

  PRGPrintf ("Clock: %d", Index);
}


void SetData (int *Data, int Index, int Signal, bool Bit)
{
  int Mask;

  Mask = 1 << Signal;  // TODO: evtl. -1 falls Index = 1..

  if (Bit)
    Data[Index] |= Mask;
  else
    Data[Index] &= ~Mask;
}  


static void AnalyzerTimer (void *Entry, XtIntervalId ID)
{
  if (Data.Pos < SAMPLES)
  {
    Step (Data.Pos++);
    Data.Timer = XtAppAddTimeOut (AppCon, Data.Delay,
				  (XtTimerCallbackProc)AnalyzerTimer, NULL);
  }
  else
    Stop ();
}


static void InitDisplay (Widget Work)
{
  Data.ScrollWin = 
    XtVaCreateManagedWidget ("signal",
			     xmScrolledWindowWidgetClass, Work,
			     XmNscrollingPolicy,          XmAUTOMATIC,
			     XmNscrollBarDisplayPolicy,   XmAS_NEEDED,
			     XmNvisualPolicy,             XmCONSTANT,
			     XmNtopAttachment,            XmATTACH_FORM,
			     XmNleftAttachment,           XmATTACH_FORM,
			     XmNrightAttachment,          XmATTACH_FORM,
			     XmNbottomAttachment,         XmATTACH_FORM,
			     NULL);
  Data.Form = 
    XtVaCreateManagedWidget ("form", 
			     xmFormWidgetClass, Data.ScrollWin,
			     NULL);
}


static void CreateChannels ()
{
  int    count;
  Widget Top;

  Top = Data.Form;
  for (count = 0; count < 12; count++)
  { 
    Data.Channel[count] = 
      new SIGNAL ("signal", SAMPLES, Data.Form, Top, SIGNAL::DISCRETE);
    Top = Data.Channel[count]->GetWidget ();
  }

  Data.Out = new int [SAMPLES];
  for (count = 0; count < SAMPLES; count++)
    Data.Out[count] = count;
  Data.In  = new int [SAMPLES];
}



static void Load ()
{
  // Load all curves + attributes (color, ...)
}


static void Save ()
{
  // Save all curves + attributes (color, ...)
}


static void Print ()
{
  // Select channel(s) and parameter
}


static void SetParam ()
{
  DIALOG *Dlg;

  Dlg = DLGNewDialog (0);

  DLGMakeRows (Dlg, 0, 2, DlgNO_LINE, 10, 20);
  DLGInsertOMenu (Dlg, 10, "port", 4, "0x278", "0x2F8", "0x378", "0x3F8", 
		  Data.LPTBase);
  DLGInsertBool (Dlg, 20, "buffered", Data.Buffered);
  // buffered (faster) / direct output
  // Background-color

  DLGOpenDialog (Dlg, "parameters", DlgNORMAL, NULL);
  if (DLGGetExitButton(Dlg) == DlgOK)
  {
    Data.LPTBase  = DLGGetOMenu (Dlg, 10);
    Data.Buffered = DLGGetBool (Dlg, 20);
  }
  DLGFreeDialog (Dlg);
}


static void ExitPrg ()
{
  // Ask for quit
  PRGExitProgram ();
}


static void Reset ()
{
  int count;

  memset (Data.In, 0, sizeof(int) * SAMPLES);
  
  for (count = 0; count < 12; count++)
  {
    Data.Channel[count]->Clear   ();
    Data.Channel[count]->SetPos  (0);
    Data.Channel[count]->Refresh ();
  }
  if (Data.Out != NULL)
      for (count = 0; count < SAMPLES; count++)
	Data.Out[count] = count;

  Data.Pos = 0;
}


static void Run (int Mode)
{
  if (Data.Timer != 0)
    return;

  if (Data.Module == NULL)
    Data.Module = new ANALYZER (BaseAddr[Data.LPTBase]);

  switch (Mode)
  {
    case 3 :
      // Wait for trigger
      // no break -> continous

     case 1 :
       AnalyzerTimer (NULL, 0);
       break;

    case 2 :
      Step (Data.Pos++);
      break;

    default :
      printf ("Invalid mode: %d\n", Mode);
  }
}


static void Stop ()
{
  int count;

  if (Data.Timer != 0)
  {
    XtRemoveTimeOut (Data.Timer);
    Data.Timer = 0;
  }

  if (Data.Buffered)
    for (count = 0; count < 12; count++)
      Data.Channel[count]->Refresh ();

  if (Data.Module != NULL)
  {
    delete Data.Module;
    Data.Module = NULL;
  }
}


static int EditCB (DIALOG *Dlg, int BoxID)
{
  int Page = DLGGetNote (Dlg, 10);
  int Channel;

  Channel = DLGGetOMenu (Dlg, 5);

  switch (Page)
  {
    case 1 :
      switch (BoxID)
      {
        case 110 :
	  Data.Channel[Channel]->SetName (DLGGetText(Dlg, BoxID));
	  break;
        case 120 : 
	  Data.Channel[Channel]->SetColor ((SIGNAL::COLOR)DLGGetOMenu(Dlg, 
								      BoxID));
	  break;
      }
      break;

    case 2 :
      break;

    case 3 :
      break;
  }

  return DlgACCEPT;
}


static void Edit ()
{
  DIALOG *Dlg;

  Dlg = DLGNewDialog (0);

  DLGMakeRows (Dlg, 0, 2, DlgNO_LINE, 5, 10);

  DLGInsertOMenu (Dlg, 5, "channel", 12, 
		          "output1",  "output2",  "output3",
		          "output4",  "output5",  "output6",
		          "output7",  "output8",  "output9",
		          "output10", "output11", "output12", 0);

  DLGInsertNote (Dlg, 10, "page", DlgNOBINDING);
    DLGAddPage (Dlg, 10, DlgMAJOR, "global", 100);
      DLGMakeRows (Dlg, 100, 5, DlgNO_LINE, 110, 120, 130, 140, 150);
      DLGMakeCols (Dlg, 150, 2, DlgNO_LINE, 151, 152);
      DLGInsertText  (Dlg, 110, "name", 20, "channel", 20, 1);
      DLGInsertOMenu (Dlg, 120, "color", 10, "black", "grey",   "white",
		                             "red",   "orange", "yellow", 
		                             "green", "blue",   "violet", 
		                             "brown", 5);
      DLGInsertInt   (Dlg, 130, "size", 2000, 5);
      DLGInsertOMenu (Dlg, 140, "zoomY", 10, "auto", NULL, 
		                "zoom1",  "zoom2",  "zoom4",  "zoom8", 
   		                "zoom16", "zoom32", "zoom64", "zoom128", 0);

      DLGInsertPic (Dlg, 151, "load", DlgXPM, -1, -1, load_xpm, NULL);
      DLGInsertPic (Dlg, 152, "save", DlgXPM, -1, -1, save_xpm, NULL);

    DLGAddPage (Dlg, 10, DlgMAJOR, "edit", 200);
      DLGMakeRows (Dlg, 200, 5, DlgNO_LINE, 210, 220, 230, 240, 250);
      DLGMakeCols (Dlg, 250, 3, DlgNO_LINE, 251, 252, 253);
      DLGInsertText  (Dlg, 210, "pattern", 80, "10101010", 20, 1);
      DLGInsertOMenu (Dlg, 220, "predef", 4, "zero", "one", "clock", 
		                             "impulse", 0);
      DLGInsertScale (Dlg, 230, "start", 1.0, 2000.0, 1.0,    
		                         0, DlgHORIZONTAL);
      DLGInsertScale (Dlg, 240, "end",   1.0, 2000.0, 2000.0, 
		                         0, DlgHORIZONTAL);
      // Paramter: clock-freq.
      DLGInsertProc (Dlg, 251, "invert", NULL);
      DLGInsertProc (Dlg, 252, "mirror", NULL);
      DLGInsertProc (Dlg, 253, "cut",    NULL);
      // Range

    DLGAddPage (Dlg, 10, DlgMAJOR, "trigger", 300);

  DLGSetFunction (Dlg, 0, EditCB);

  DLGMakeButton (Dlg, OK_BUTTON);
  DLGOpenDialog (Dlg, "editor", DlgNORMAL, NULL);

  DLGFreeDialog (Dlg);

  // TODO:
  // Bitmuster generator: Clear, Set, Muster, Toggle, Value, Mirror, Cut, Move
  // Predefined: clock, impulse
  // Trigger (Input/output: raise, fall, bitmuster, clock-tick)
}


static void SetZoom (int ZoomFactor)
{
  int count;

  for (count = 0; count < 12; count++)
    Data.Channel[count]->SetZoomX (ZoomFactor);
}


void Options ()
{
  DIALOG *Dlg;

  Dlg = DLGNewDialog (0);

  DLGMakeRows (Dlg,  0, 4, DlgNO_LINE,   10, 50);
  DLGMakeCols (Dlg, 10, 2, DlgETCHED_IN, 20, 30);
  DLGMakeRows (Dlg, 20, 2, DlgNO_LINE,   21, 22);
  DLGMakeRows (Dlg, 30, 2, DlgNO_LINE,   31, 32);
  DLGMakeCols (Dlg, 50, 2, DlgNO_LINE,   55, 60);
  DLGInsertUpDn (Dlg, 21, "startPos", 1.0, 1.0,    1.0, 1);
  DLGInsertUpDn (Dlg, 22, "endPos",   1.0, 1.0, 2048.0, 1);
  DLGInsertUpDn (Dlg, 31, "delay",    0.0, 0.0,    Data.Delay, 1);

  // absolute timing (send each xxx msec 
  DLGInsertTBox (Dlg, 55, "outSelect", 2, 12,
		          "output1",  TRUE,  "output2",  TRUE,
		          "output3",  TRUE,  "output4",  TRUE,
		          "output5",  TRUE,  "output6",  TRUE,
		          "output7",  TRUE,  "output8",  TRUE,
		          "output9",  TRUE,  "output10", TRUE,
		          "output10", TRUE,  "output12", TRUE);
  DLGInsertTBox (Dlg, 60, "inSelect", 2, 12,
		          "input1",   TRUE,  "input2",   TRUE,
		          "input3",   TRUE,  "intput4",  TRUE,
		          "input5",   TRUE,  "input6",   TRUE,
		          "input7",   TRUE,  "intput8",  TRUE,
		          "input9",   TRUE,  "input10",  TRUE, 
		          "input11",  TRUE,  "intput12", TRUE);

  // Global trigger

  DLGOpenDialog (Dlg, "options", DlgNORMAL, NULL);
  if (DLGGetExitButton(Dlg) == DlgOK)
  {
    Data.Delay = int (DLGGetUpDn(Dlg, 31));
  }

  DLGFreeDialog (Dlg);
}


void Eject ()  // Eject card
{
  struct timeval Start, End;
         CARD    Card (BaseAddr[Data.LPTBase]);

  PRGPrintf ("Insert card...");
  while (! Card.Inserted())
    TOOLWait (100000);

  PRGPrintf ("Load card...");
  while (! Card.Loaded())
    TOOLWait (100000);

  PRGPrintf ("Ejecting card in 2 second...");
  TOOLWait (2000000);

  gettimeofday (&Start, NULL);

  Card.Eject ();

  gettimeofday (&End, NULL);  

  printf ("SLND: %.3f sec\n", 
	  0.000001 * ((End.tv_sec-Start.tv_sec) * 1000000 + 
		      (End.tv_usec-Start.tv_usec)));

  PRGPrintf ("Remove card...");
  while (Card.Inserted())
    TOOLWait (100000);

  PRGPrintf ("finished !");
}


void Evaluate ()  // Read and show card data
{
  int  TrackNo;
  int  ClockMask[3] = {0x001, 0x004, 0x010};
  int  DataMask[3]  = {0x002, 0x008, 0x020};
  int  count, Index;
  int  Value, Info;
  int  Last[3], No[3] = {0, 0, 0};
  int  Track[3][SAMPLES];
  CARD Card (BaseAddr[Data.LPTBase]);

  PRGPrintf ("Insert card...");
  while (! Card.Inserted())
    TOOLWait (100000);

  PRGPrintf ("Load card...");
  while (! Card.Loaded())
    TOOLWait (100000);

  for (count = 5; count > 0; count--)
  {
    PRGPrintf ("Ejecting card in %d seconds...", count);
    TOOLWait (1000000);      // Wait 1 seconds 
  }
  Card.Eject ();

  PRGPrintf ("Reading...");

  Index = 0;
  Last[0] = Last[1] = Last[3] = 0;
  while (Card.Inserted())
  {
    Value = Card.Read (/* TRACK_1 | */ TRACK_2 | TRACK_3, 2);

    for (TrackNo = 0; TrackNo < 3; ++TrackNo)
    {
      // Trigger on falling clock
      if ((Last[TrackNo] & ClockMask[TrackNo]) < (Value & ClockMask[TrackNo]))
      {
	// Extract bit of track (inverted)
	Info = ((Value & DataMask[TrackNo]) == 0);

	Data.Channel[2*TrackNo]->AddValue   (1);
	Data.Channel[2*TrackNo+1]->AddValue (Info);
        Data.Channel[2*TrackNo]->AddValue   (0);
	Data.Channel[2*TrackNo+1]->AddValue (Info);

	Track[TrackNo][No[TrackNo]] = Info;

	Data.Channel[2*TrackNo]->Refresh   ();
        Data.Channel[2*TrackNo+1]->Refresh ();
	(No[TrackNo])++;

	PRGPrintf ("Values/Track: %d %d %d...", No[0], No[1], No[2]);
      }
      Last[TrackNo] = Value;
    }
  }

  PRGPrintf ("\\finished");

  /* write track data */
  for (TrackNo = 0; TrackNo < 3; ++TrackNo)
  {
    printf ("Track %d: #%d\n", TrackNo+1, No[TrackNo]);
    count = 0;
    while (count+4 < No[TrackNo])
    {
      Value =   (Track[TrackNo][count  ] << 3)
	      + (Track[TrackNo][count+1] << 2)
	      + (Track[TrackNo][count+2] << 1)
   	      +  Track[TrackNo][count+3];
      printf ("%d %d %d %d (%d) = %c\n", 
	      Track[TrackNo][count],   Track[TrackNo][count+1],
	      Track[TrackNo][count+2], Track[TrackNo][count+3],
	      Track[TrackNo][count+4], Bit4Code[Value]);
      count += 5;
    }
  }
}


#if 0
    for (TrackNo = 0; TrackNo < 6; TrackNo++)
    {
      Data.Channel[TrackNo]->AddValue (Value & 1);
      Data.Channel[TrackNo]->Refresh ();
      Value >>= 1;                      // Shift right one bit
    }
#endif


void CardInfo ()
{
  CARD Card (BaseAddr[Data.LPTBase]);

  PRGPrintf ("Insert Card...");
  while (! Card.Inserted())
    ;

  PRGPrintf ("Inserted: %s", Card.Inserted()? "YES":"NO");
  for (int count = 0; count < 1000; ++count)
  {
    while (Card.Inserted())
      Card.Read (/* TRACK_2 | TRACK_3 */0xfff, 3);
    printf ("OUT: %d\n", count);
  }
  
  PRGPrintf ("finished\n");

#if 0
  DLGOpenTextDialog ("cardInfo", DlgINFO,
		     "This functions are designed for a\n"
		     "Panasonic ZU-95130SE1 card reader", 1, "OK");
#endif
  // TODO: Anschlussbelegung beschreiben
}



void ModuleTest ()
{
  int  count = -1, Value = 0;
  char Msg[64];

  Data.Module = new ANALYZER (BaseAddr[Data.LPTBase]);

  PRGPrintf ("Module test...");

  if (DLGOpenTextDialog("moduleTest", DlgINFO, 
			"Connect output- and input-pins 1:1 for this test",
			2, "ABORT", "OK") == 1)
  {
    PRGPrintf ("\\Function aborted");
    return;
  }

  do
  {
    count++;
    Data.Module->Write (count);
    Value = Data.Module->Read ();
  } while (count < 0x0FFF && Value == count);

  if (Value != count)
  {
    sprintf (Msg, "Error at data %d: value 0x%04X\n"
	          "Check power supply and cabling !",
	          count, Value);
    DLGOpenTextDialog ("moduleError", DlgERROR, Msg, 1, "OK");
  }
  else
    PRGPrintf ("\\done !");

  delete (Data.Module);
  Data.Module = NULL;
}


void SpeedTest ()
{
         char          Msg[128];
         unsigned long Time1, Time2;
  struct timeval       Start, End;

  Data.Module = new ANALYZER (BaseAddr[Data.LPTBase]); 

  PRGPrintf ("Speed test - please wait...");
 
  // Measuer output speed: about 82.000 12-bit values per second (82 kHz)
  gettimeofday (&Start, NULL);
  for (int count = 0; count <= 100000; count++)
    Data.Module->Write (count % 0x0FFF);
  gettimeofday (&End, NULL);
  Time1 = (End.tv_sec-Start.tv_sec) * 1000000 + (End.tv_usec-Start.tv_usec);

  // Measure input speed: about 10.000 12-bit samples per second (10 kHz)
  gettimeofday (&Start, NULL);
  for (int count = 0; count <= 100000; count++)
    Data.Module->Read ();
  gettimeofday (&End, NULL);
  Time2 = (End.tv_sec-Start.tv_sec) * 1000000 + (End.tv_usec-Start.tv_usec);

  delete (Data.Module);
  Data.Module = NULL;

  PRGPrintf ("\\done");

  sprintf (Msg, "Analyzer Speed test:\n"
   	        "Write Speed: %.0f 12-bit values / second\n"
	        "Read  Speed: %.0f 12-bit samples / second",
	        1000000.0 * 1000000.0 / Time1,
	        1000000.0 * 1000000.0 / Time2);
  DLGOpenTextDialog ("speedTest", DlgINFO, Msg, 1, "OK");

}


void Manual ()
{
}


int main (int argc, char *argv[])
{
  Widget Work;

  static MENU_ENTRY SystemMenu[] = 
  {
    ENTRY ("load",      "",            "",      ' ', TRUE, Load,      NULL),
    ENTRY ("save",      "",            "",      ' ', TRUE, Save,      NULL),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("print",     "",            "",      ' ', TRUE, Print,     NULL),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("parameter", "",            "",      ' ', TRUE, SetParam,  NULL),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("exit",      "Alt<Key>F4", "Alt+F4", 'x', TRUE, ExitPrg,   NULL)
  };

  static MENU_ENTRY ZoomMenu[] = 
  {
    ENTRY ("x1",      "",       "",        ' ', TRUE, SetZoom,   1),
    ENTRY ("x2",      "",       "",        ' ', TRUE, SetZoom,   2),
    ENTRY ("x4",      "",       "",        ' ', TRUE, SetZoom,   4),
    ENTRY ("x8",      "",       "",        ' ', TRUE, SetZoom,   8),
    ENTRY ("x16",     "",       "",        ' ', TRUE, SetZoom,  16),
    ENTRY ("x32",     "",       "",        ' ', TRUE, SetZoom,  32),
    ENTRY ("x64",     "",       "",        ' ', TRUE, SetZoom,  64),
    ENTRY ("x128",    "",       "",        ' ', TRUE, SetZoom, 128)
  };

  static MENU_ENTRY AnalyzerMenu[] = 
  {
    ENTRY ("edit",      "",      "",        ' ', TRUE, Edit,     NULL),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("reset",   "",        "",        ' ', TRUE, Reset,    NULL),
    ENTRY ("run",     "",        "",        ' ', TRUE, Run,         1),
    ENTRY ("step",    "",        "",        ' ', TRUE, Run,         2),
    ENTRY ("stop",    "",        "",        ' ', TRUE, Stop,     NULL),
    MENU_LINE (PrgETCHED_IN), 
    SUB_MENU ("zoom",   "",      "",        ' ', TRUE, ZoomMenu),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("options",   "",      "",        ' ', TRUE, Options,  NULL)
  };

  static MENU_ENTRY CardMenu[] = 
  {
    ENTRY ("eject",    "",        "",       ' ', TRUE,  Eject,     NULL),
    ENTRY ("evaluate", "",        "",       ' ', TRUE,  Evaluate,  NULL),
    MENU_LINE (PrgETCHED_IN),
    ENTRY ("cardInfo", "",        "",       ' ', TRUE,  CardInfo,  NULL)
  };

  static MENU_ENTRY TestMenu[] = 
  {
    ENTRY ("module",    "",        "",      ' ', TRUE, ModuleTest, NULL),
    ENTRY ("speed",     "",        "",      ' ', TRUE, SpeedTest,  NULL)
  };

  static MENU_ENTRY HelpMenu[] = 
  {
  };

  static MENU_COLUMN MenuCols[] =
  {
    FUNC_MENU ("system", 's', TRUE, SystemMenu),
    FUNC_MENU ("module", 'm', TRUE, AnalyzerMenu),
    FUNC_MENU ("card",   'c', TRUE, CardMenu),
    FUNC_MENU ("test",   't', TRUE, TestMenu),
    HELP_MENU ("help",   'h', TRUE, HelpMenu)
  };

  static MENU_BAR Menu = MAIN_MENU ("mainMenu", MenuCols);


  PRGInitLib (argv[0], &argc, argv, NULL);

  PRGMakeToolBar (PrgTOP_TOOL);
  PRGAddToolIcon (-1, -1, bomb_xpm,       ExitPrg,       NULL, "Exit program");
  PRGAddToolSpace (5);
  PRGAddToolIcon (-1, -1, load_xpm,       Load,          NULL, "Load data");
  PRGAddToolIcon (-1, -1, save_xpm,       Save,          NULL, "Save data");
  PRGAddToolSpace (10);
  PRGAddToolIcon (-1, -1, tv_xpm,         Reset,         NULL, "Reset");
  PRGAddToolIcon (-1, -1, plot_xpm,  (MENU_FUNCTION)Run, (void *)1, "Run");
  PRGAddToolIcon (-1, -1, plot_xpm,  (MENU_FUNCTION)Run, (void *)2, "Step");
  PRGAddToolIcon (-1, -1, stop_xpm,       Stop,          NULL,      "Stop");
  PRGAddToolSpace (10);
  PRGAddToolIcon (-1, -1, tv_xpm,         Eject,         NULL,  "Eject Card");
  PRGAddToolIcon (-1, -1, card_xpm,       Evaluate,      NULL,  "Read Card");
  PRGAddToolIcon (-1, -1, card_xpm,       CardInfo,      NULL,  "Card Info");
  PRGAddToolSpace (15);
  PRGAddToolIcon (-1, -1, help2_xpm,      Manual,        NULL,  "Manual");

  Work = PRGCreateProgram (argv[0], &Menu);
  InitDisplay (Work);

  PRGSetSize (-1, 480);
  PRGStartProgram (CreateChannels, NULL);

  return 0;
}
